package PktDump::Analyser::ARP;

use strict;
use warnings;

use base qw( PktDump::Analyser );

use constant ETHERTYPE => 0x0806;

my %opcodes = (
   1 => "REQUEST",
   2 => "REPLY",
   3 => "REQUEST REVERSE",
   4 => "REPLY REVERSE",
);

sub analyse
{
   my $self = shift;
   my ( $pkt, $opts ) = @_;

   my ( $htype, $ptype, $hlen, $plen, $opcode, $addrs ) = unpack(
       "n       n       C      C      n        a*", $pkt );

    my ( $sha, $spa, $tha, $tpa ) = unpack( "a$hlen a$plen a$hlen a$plen", $addrs );

    $_ = $self->format_hwaddr  ( $htype, $_, $opts ) for $sha, $tha;
    $_ = $self->format_protaddr( $ptype, $_, $opts ) for $spa, $tpa;

    sprintf( "ARP| %s %s %s %s %s", $opcodes{$opcode}||$opcode,
       $sha, $spa, $tha, $tpa );
}

sub handle_filter_arp
   :Filter_description(Address Resolution Protocol)
{
   my $self = shift;

   my $below = $self->build_layer_filter( proto => ETHERTYPE );

   my @parts = $below;

   return PktDump::Filter->all( @parts );
}

1;
