package PktDump::Analyser::CHAP;

use strict;
use warnings;

use base qw( PktDump::Analyser );

use constant PPPDLL_PROTO => 0xc223;

# See also
#   http://www.faqs.org/rfcs/rfc1994.html

sub analyse
{
   my $self = shift;
   my ( $pkt, $opts ) = @_;

   my ( $code, $id, $len ) = unpack( "C C n", $pkt );
   my $data = substr( $pkt, 4, $len - 4 );

   my @lines;
   if( my $mth = $self->can( sprintf "analyse_%02x", $code ) ) {
      push @lines, $mth->( $self, $data, $opts );
   }
   else {
      push @lines, sprintf "CHAP| code=%02x id=%02x data={%d bytes}", $code, $id, length $data;
      push @lines, $self->format_hex( $data ) if $opts->{hexdump};
   }

   return $opts->{one_line} ? $lines[0] : @lines;
}

sub analyse_cr
{
   my $self = shift;
   my ( $cr, $data, $opts ) = @_;

   ( my $valsize, $data ) = unpack( "C a*", $data );
   my $value = substr( $data, 0, $valsize, "" );
   my $name = $data;

   return "CHAP| $cr name=$name [" . unpack( "H*", $value ) . "]";
}

sub analyse_01
{
   my $self = shift;
   return $self->analyse_cr( "Challenge", @_ );
}

sub analyse_02
{
   my $self = shift;
   return $self->analyse_cr( "Response", @_ );
}

sub analyse_03
{
   my $self = shift;
   my ( $data, $opts ) = @_;

   return "CHAP| Success: $data";
}

sub analyse_04
{
   my $self = shift;
   my ( $data, $opts ) = @_;

   return "CHAP| Failure: $data";
}

1;
