package PktDump::Analyser::ESP;

use strict;
use warnings;

use base qw( PktDump::Analyser );

use constant IPPROTO => 50;

use PktDump::Filter;

sub analyse
{
   my $self = shift;
   my ( $pkt, $opts, $outer ) = @_;

   my ( $spi, $seq, $data ) = unpack( "N N a*", $pkt );

   my $line = sprintf "spi=0x%08x seq=%d", $spi, $seq;

   if( $opts->{one_line} ) {
      return sprintf("ESP| %s->%s ", $outer->{src}, $outer->{dst}) . $line;
   }
   else {
      return "ESP| $line";
   }

   my @lines = "UDP| $line";
   push @lines, $self->format_hex( $data ) if $opts->{hexdump};

   return @lines;
}

sub handle_filter_esp
   :Filter_description(IPsec's Encapsulating Security Protocol)
   :Filter_option(spi => 'number')
{
   my $self = shift;
   my ( $opts ) = @_;

   # 'esp' on its own really means ip proto ...
   my @parts = my $below = $self->parse_filter( $_ = "ip proto ".IPPROTO );

   if( my $spi = $opts->{spi} ) {
      push @parts, $below->match_u32( 0, $spi );
   }

   return PktDump::Filter->all( @parts );
}

1;
