package PktDump::Analyser::Ether802dot1Q;

use strict;
use warnings;

use base qw( PktDump::Analyser::Ethernet );

use constant ETHERTYPE => 0x8100;

sub analyse
{
   my $self = shift;
   my ( $pkt, $opts ) = @_;

   my ( $tag, $ethertype, $data ) = unpack( "n n a*", $pkt );

   my $pcp = ( $tag & 0xe000 ) >> 13;
   my $vid = ( $tag & 0x0fff );

   my $linkline = "802.1Q| vid=$vid";
   $linkline .= " prio=$pcp" if $opts->{verbose};

   if( $opts->{link_header} ) {
      return $self->analyse_ethernet( $linkline, $ethertype, $data, $opts );
   }
   elsif( $opts->{one_line} ) {
      return "$linkline " . ( $self->analyse_ethernet( "", $ethertype, $data, $opts ) )[0];
   }
   else {
      return $linkline, $self->analyse_ethernet( "", $ethertype, $data, $opts );
   }
}

sub handle_filter_vlan
   :Filter_description(Ethernet 802.1Q VLAN)
   :Filter_option(vid => 'numbers')
{
   my $self = shift;
   my ( $opts ) = @_;

   my @parts = my $below = $self->build_layer_filter( proto => ETHERTYPE );

   if( my $vid = $opts->{vid} ) {
      push @parts, $below->match_u16( 0, $vid, mask => 0x0fff );
   }

   return PktDump::Filter->all( @parts );
}

1;
