package PktDump::Analyser::Ethernet;

use strict;
use warnings;

use base qw( PktDump::Analyser );

sub analyse_ethernet
{
   my $self = shift;
   my ( $linkline, $ethertype, $data, $opts ) = @_;

   my $inner_analyser = eval { $self->for_ethertype( $ethertype ) };
   if( $@ and $@ =~ m/^Unable to find an Analyser for ETHERTYPE / ) {
      if( $opts->{one_line} ) {
         return sprintf "Ethernet| %s type=0x%04x", $linkline, $ethertype;
      }
      else {
         my @lines;
         push @lines, $self->format_hex( $data ) if $opts->{hexdump};
         return $linkline, @lines;
      }
   }
   elsif( $@ ) {
      die $@;
   }

   if( $opts->{link_header} ) {
      if( $opts->{one_line} ) {
         return "$linkline " . ( $inner_analyser->analyse( $data, $opts ) )[0];
      }
      else {
         return $linkline, $inner_analyser->analyse( $data, $opts );
      }
   }
   else {
      return $inner_analyser->analyse( $data, $opts );
   }
}

sub format_mac
{
   my $self = shift;
   my ( $addr, $opts ) = @_;
   join ":", map sprintf("%02X",$_), unpack( "C C C C C C", $addr );
}

1;
