package PktDump::Analyser::ICMPv6;

use strict;
use warnings;

use base qw( PktDump::Analyser );

use constant IPPROTO => 58;

# See also
#   http://www.faqs.org/rfcs/rfc2463.html

sub analyse
{
   my $self = shift;
   my ( $pkt, $opts, $outer ) = @_;

   my ( $type, $code, $cksum, $data ) = unpack(
       "C      C      n       a*", $pkt );

   my $hosts = "";
   if( $opts->{one_line} ) {
      $hosts = "$outer->{src}->$outer->{dst} ";
   }

   if( my $mth = $self->can( "analyse_icmp_$type" ) ) {
      "ICMPv6| ".$hosts . $mth->( $self, $opts, $code, $data );
   }
   else {
      sprintf "ICMPv6| %stype=%d code=%d", $hosts, $type, $code;
   }
}

sub analyse_icmp_128 { &analyse_icmp_echo( shift, "echo-request", @_ ) }
sub analyse_icmp_129 { &analyse_icmp_echo( shift, "echo-reply", @_ ) }
sub analyse_icmp_echo
{
   my $self = shift;
   my ( $name, $opts, $code, $data ) = @_;
   my ( $id, $seq ) = unpack( "n n", $data );
   return "$name seq=$seq" . ( $opts->{verbose} ? " id=$id" : "" );
}

1;
