package PktDump::Analyser::PPP_IPCP;

use strict;
use warnings;

use base qw( PktDump::Analyser );

use constant PPPDLL_PROTO => 0x8021;

# See also
#   http://www.faqs.org/rfcs/rfc1332.html
#   http://www.faqs.org/rfcs/rfc1877.html

sub analyse
{
   my $self = shift;
   my ( $pkt, $opts ) = @_;

   my ( $code, $id, $len, $data ) = unpack( "C C n a*", $pkt );

   if( my $mth = $self->can( sprintf "analyse_%d", $code ) ) {
      return "IPCP| " . $mth->( $self, $code, $id, $data );
   }
   else {
      return sprintf "IPCP| code=%02x id=%02x data={%v02x}", $code, $id, $data;
   }
}

my %_format_option = (
   1   => sub { "ipaddrs=[".unpack("H*",$_)."]" },
   2   => sub { "compression=".sprintf("%04x",unpack("n",$_)) },
   3   => sub { "address=".sprintf("%vd",$_) },
   129 => sub { "dns1=".sprintf("%vd",$_) },
   130 => sub { "nbns1=".sprintf("%vd",$_) },
   131 => sub { "dns2=".sprintf("%vd",$_) },
   132 => sub { "nbns2=".sprintf("%vd",$_) },

   default => sub { sprintf "[%d=%v02x]", $_[0], $_[1] },
);

sub _format_options
{
   my $self = shift;
   my ( $data ) = @_;

   my @opts;
   while( length $data ) {
      ( my $type, my $len, $data ) = unpack( "C C a*", $data );
      last if $type == 0;

      my $value = substr( $data, 0, $len - 2, "" );

      push @opts, ( $_format_option{$type} || $_format_option{default} )->( $type, local $_ = $value );
   }

   return join " ", @opts;
}

sub analyse_1
{
   return "Configure-Request " . $_[0]->_format_options( $_[3] );
}

sub analyse_2
{
   return "Configure-Ack " . $_[0]->_format_options( $_[3] );
}

sub analyse_3
{
   return "Configure-Nak " . $_[0]->_format_options( $_[3] );
}

sub analyse_4
{
   return "Configure-Reject " . $_[0]->_format_options( $_[3] );
}

sub analyse_5
{
   return sprintf "Terminate-Request %v02x", $_[3];
}

sub analyse_6
{
   return sprintf "Terminate-Ack %v02x", $_[3];
}

sub analyse_7
{
   return sprintf "Code-Reject %v02x", $_[3];
}

1;
