package PktDump::Analyser::PPPoEdiscovery;

use strict;
use warnings;

use base qw( PktDump::Analyser );

use constant ETHERTYPE => 0x8863;

# See also
#   http://www.faqs.org/rfcs/rfc2516.html

sub analyse
{
   my $self = shift;
   my ( $pkt, $opts ) = @_;

   my ( $vertype, $code, $session, $len, $data ) = unpack( "C C n n a*", $pkt );
   $vertype == 0x11 or die "Unrecognised PPPoE version type";

   my @lines;

   if( my $mth = $self->can( sprintf "analyse_%02x", $code ) ) {
      push @lines, $mth->( $self, $opts, $session, substr( $data, 0, $len ) );
   }
   else {
      push @lines, sprintf "PPPoE| discovery code=%02x session=%04x", $code, $session;
      push @lines, $self->format_hex( $pkt ) if $opts->{hexdump};
   }

   return $opts->{one_line} ? $lines[0] : @lines;
}

my %_format_tag = (
   0x101, sub { "Service-Name:$_" },
   0x102, sub { "AC-Name:$_" },
   0x103, sub { "Host-Uniq:".pack("H*",$_) },
   0x104, sub { "AC-Cookie:".pack("H*",$_) },
   0x110, sub { "Relay-Session-Id:".pack("H*",$_) },
   0x201, sub { "Service-Name-Error:$_" },
   0x202, sub { "AC-System-Error:$_" },
   0x203, sub { "Generic-Error:$_" },
);

sub format_tags
{
   my $self = shift;
   my ( $data ) = @_;

   my $ret = "";

   while( length $data ) {
      ( my $type, my $len, $data ) = unpack( "n n a*", $data );
      my $value = substr( $data, 0, $len, "" );

      $ret .= " [". ( $_format_tag{$type} || sub { sprintf "%04x:%v02x", $type, $_ } )->( local $_ = $value ) . "]";
   }

   return $ret;
}

sub analyse_09
{
   my $self = shift;
   my ( $opts, $session, $data ) = @_;

   return "PPPoE| PADI" . $self->format_tags( $data );
}

sub analyse_07
{
   my $self = shift;
   my ( $opts, $session, $data ) = @_;

   return "PPPoE| PADO" . $self->format_tags( $data );
}

sub analyse_19
{
   my $self = shift;
   my ( $opts, $session, $data ) = @_;

   return "PPPoE| PADR" . $self->format_tags( $data );
}

sub analyse_65
{
   my $self = shift;
   my ( $opts, $session, $data ) = @_;

   return "PPPoE| PADS" . $self->format_tags( $data );
}

sub analyse_a7
{
   my $self = shift;
   my ( $opts, $session, $data ) = @_;

   return "PPPoE| PADT" . $self->format_tags( $data );
}

1;
