package PktDump::Analyser::PcapLinuxSLL;

# This has virtually nothing to do with the real Linux sockaddr_sll
# This is simply what libpcap's ancient idea -thinks- of sockaddr_sll

use strict;
use warnings;

use base qw( PktDump::Analyser );

use constant PCAP_DLT => 113;

use Socket::Packet qw( /^PACKET_/ );

sub write_for_pcap
{
   my $self = shift;
   my ( $packet, $info ) = @_;

   my $analyser = $info->{analyser};
   # pcap is expecting packets missing their link header here

   my $hdrlen = $analyser->hdrlen( $info, $packet );

   my $dir   = $info->{dir};
   my $scope = $info->{scope};
   my $pkttype = ( $dir eq "TX"  ) ? PACKET_OUTGOING :
                 ( $scope eq "U" ) ? PACKET_HOST :
                 ( $scope eq "M" ) ? PACKET_MULTICAST :
                 ( $scope eq "B" ) ? PACKET_BROADCAST :
                 ( $scope eq "O" ) ? PACKET_OUTGOING :
                                     -1;

   return pack( "n n n a8 n a*",
      $pkttype, $analyser->HATYPE, length($info->{addr}), $info->{addr}, $info->{ethertype},
      substr( $packet, $hdrlen ),
   );
}

sub read_from_pcap
{
   my $self = shift;
   my ( $data, $info ) = @_;

   ( $info->{pkttype}, my $hatype, my $addrlen, $info->{addr}, $info->{ethertype}, my $packet ) =
      unpack( "n n n a8 n a*", $data );

   substr( $info->{addr}, $addrlen ) = "";

   # This is a massive cheat
   %$info = ( %$info, PktDump::PacketSocket->_pkttype2info( $info->{pkttype} ) );

   return $packet;
}

sub analyse
{
   my $self = shift;
   my ( $pkt, $opts, $info ) = @_;

   return PktDump::Analyser->for_ethertype( $info->{ethertype} )->analyse( $pkt, $opts, $info );
}

1;
