package PktDump::Capfile;

use strict;
use warnings;

use Carp;

use base qw( PktDump::PacketSourceRole );

use Module::Pluggable
   search_path => [ "PktDump::Capfile" ],
   require     => 1;

# Subclass search key
sub handles_format { my $self = shift; $self->FORMAT eq shift() }

sub new
{
   my $class = shift;
   my ( $dir, $path ) = @_;

   if( $class eq __PACKAGE__ ) {
      my $format;

      $format or 
         $path =~ s/^(\w+):// and $format = $1;

      $format or
         $path =~ m/\.(\w+)$/ and $format = $1;

      $format ||= "pcap";

      undef $class;
      $_->handles_format( $format ) and $class = $_, last for plugins;

      defined $class or die "Cannot find a PktDump::Capfile implementation for '$format'\n";
   }

   my $fh;
   if( $dir eq "write" ) {
      $class->can( "write_packet" ) or
         die $class->FORMAT . " cannot be used for writing capture files\n";

      open $fh, ">", $path or die "Cannot write $path - $!\n";
   }
   elsif( $dir eq "read" ) {
      $class->can( "read_packet" ) or
         die $class->FORMAT . " cannot be used for reading capture files\n";

      open $fh, "<", $path or die "Cannot read $path - $!\n";
   }
   else {
      die "Unknown capfile direction $dir\n";
   }

   $fh->autoflush( 1 );

   my $self = bless {
      fh  => $fh,
      dir => $dir,
   }, $class;

   if( $dir eq "write" ) {
      $self->write_header;
   }
   elsif( $dir eq "read" ) {
      $self->read_header;
   }

   return $self;
}

sub pack
{
   my $self = shift;
   my ( $fmt, @args ) = @_;

   $self->{dir} eq "write" or croak "Cannot pack except into write-mode Capfiles";

   $self->{fh}->print( pack( $fmt, @args ) );
}

sub unpack
{
   my $self = shift;
   my ( $fmt, $len ) = @_;

   defined $self->{fh}->read( my $buffer, $len )
      or die "Cannot read from file - $!";

   length $buffer or return; # EOF

   my @values = unpack( "$fmt .", $buffer );
   pop @values == $len or croak "unpack('$fmt') didn't take $len bytes";

   return @values;
}

1;
