package PktDump::Capfile::Pcap;

use strict;
use warnings;

use base qw( PktDump::Capfile );
use constant FORMAT => "pcap";

use constant PCAP_MAGIC => 0xa1b2c3d4;

# See also
#   http://www.tcpdump.org/pcap3_man.html
#   http://www.tcpdump.org/linktypes.html

sub write_header
{
   my $self = shift;

   my $dlt = 113;

   $self->pack( "I s s i i i i",
      PCAP_MAGIC,
      2, 4,  # version
      0,     # GMT offset
      6,     # sigfigs
      65535, # snaplen
      $dlt,  # Linux SLL
   );

   $self->{analyser} = PktDump::Analyser->for_pcap_dlt( $dlt );
}

sub read_header
{
   my $self = shift;

   my ( $magic, $vermajor, $verminor, undef, $sigfigs, undef, $dlt ) =
      $self->unpack( "I s s i i i i", 24 );

   die "Bad magic - $magic" unless $magic == PCAP_MAGIC;
   die "Bad version - $vermajor.$verminor" unless $vermajor == 2 and $verminor == 4;

   $self->{analyser} = PktDump::Analyser->for_pcap_dlt( $dlt );
}

sub write_packet
{
   my $self = shift;
   my ( $info, $packet ) = @_;

   my $data = $self->{analyser}->write_for_pcap( $packet, $info );

   my $len = length $data;

   $self->pack( "i i i i", $info->{ts_sec}, $info->{ts_usec}, $len, $len ); # pcap

   $self->pack( "a*", $data );
}

sub read_packet
{
   my $self = shift;

   my ( $ts_sec, $ts_usec, $caplen, $len ) = $self->unpack( "i i i i", 16 )
      or return undef;

   my $analyser = $self->{analyser};

   my %info = (
      analyser => $analyser,

      ts_sec  => $ts_sec,
      ts_usec => $ts_usec,

      len    => $len,
      caplen => $caplen,

      ifindex => -1,
      ifname  => "?",
   );

   my ( $data ) = $self->unpack( "a*", $caplen );

   $_[0] = $analyser->read_from_pcap( $data, \%info );

   return \%info;
}

1;
