package PktDump::OptType;

require NEXT;

sub new
{
   my $class = shift;
   return bless { @_ }, $class;
}

sub desc
{
   my $self = shift;
   return '$' . uc $self->{name};
}

sub desc_arg
{
   my $self = shift;
   return $self->{name} . " " . $self->desc;
}

sub extract_name
{
   my $self = shift;

   my $name = $self->{name};

   return $_[0] =~ s/^\Q$name\E\s*//;
}

sub extract_value
{
   my $self = shift;

   my $name = $self->{name};

   $_[0] =~ s/^([^ )]+)\s*// or
      die "Expected option $name value; found >> $_[0]\n";
   my $value = $1;

   if( my $valmatch = eval { $self->match } ) {
      $value =~ $valmatch or
         die "Expected option $name value matching $valmatch; found >> $value\n";
   }

   $value = $self->parse( local $_ = $value ) if $self->can( "parse" );

   return $value;
}

sub extract_option
{
   my $self = shift;

   return undef unless $self->extract_name( $_[0] );

   return $self->extract_value( $_[0] );
}

package PktDump::OptType::MultiValue;
use base qw( PktDump::OptType );

sub desc
{ 
   my $self = shift;
   return '$' . uc $self->{name} . '[,$' . uc $self->{name} . ',...]';
}

sub parse
{ 
   my $self = shift;
   [ map $self->parse_one($_), split m/,/, $_ ]
}

package PktDump::OptType::Directional;

sub desc_arg
{
   my $self = shift;
   return "[src|dst|local|remote] " . $self->NEXT::desc_arg;
}

sub extract_option
{
   my $self = shift;

   # We might fail; work on a temporary copy first
   my $trial = $_[0];

   my $dir;
   $trial =~ s/^(src|dst|local|remote)\s+// and $dir = $1;

   defined( my $value = $self->NEXT::extract_option( $trial ) ) or return;

   $_[0] = $trial;
   return [ $dir, $value ];
}

1;
