package PktDump::Pack64;

use strict;
use warnings;

use Exporter qw( import );
our @EXPORT_OK = qw(
   $fmt_u64 $fmt_s64
   pack_u64 unpack_u64 pack_s64 unpack_s64
);

# Not all perls can pack 'q'
our ( $fmt_u64, $fmt_s64 );

if( eval { pack "q", 0 } ) {
   $fmt_u64 = "Q";
   $fmt_s64 = "q";

   *pack_u64 = *unpack_u64 = *pack_s64 = *unpack_s64 = sub { $_[0] };
}
else {
   require Math::BigInt;
   $fmt_u64 = "a8";
   $fmt_s64 = "a8";

   *_wswap = ( unpack "C", pack "I", 1 ) ? sub { $_[1], $_[0] }
                                         : sub { $_[0], $_[1] };

   *pack_u64 = sub {
      my $hi = int( $_[0] / 2**32 );
      my $lo = $_[0] & 0xffffffff;
      pack "II", _wswap( $lo, $hi );
   };
   *unpack_u64 = sub {
      my ( $lo, $hi ) = _wswap( unpack "II", $_[0] );
      return $lo if $hi == 0;
      my $n = Math::BigInt->new( $hi ); $n <<= 32; $n |= $lo;
      return $n;
   };

   *pack_s64 = sub {
      my $hi = int( $_[0] / 2**32 );
      my $lo = $_[0] & 0xffffffff;
      pack "II", _wswap( $lo, $hi );
   };
   *unpack_s64 = sub {
      my ( $lo, $hi ) = _wswap( unpack "II", $_[0] );
      return $lo if $hi == 0;
      $hi -= 2**32 if $hi & 0x10000000;
      my $n = Math::BigInt->new( $hi ); $n <<= 32; $n |= $lo;
      return $n;
   };
}

1;
